==> jena-core-2.7.3-sources.jar: com/hp/hpl/jena/reasoner/rulesys/builtins/CountLiteralValues.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hp.hpl.jena.reasoner.rulesys.builtins;

import com.hp.hpl.jena.reasoner.rulesys.*;
import com.hp.hpl.jena.graph.*;
import java.util.*;

/**
 * CountLiteralValues(X, P, C) sets C to be the number of semantically
 * distinct values for P on resource X. This is expensive.
 * 
 * @author <a href="mailto:der@hplb.hpl.hp.com">Dave Reynolds</a>
 * @version $Revision: 1.1 $ on $Date: 2009-06-29 08:55:36 $
 */
public class CountLiteralValues extends BaseBuiltin {

    /**
     * Return a name for this builtin, normally this will be the name of the 
     * functor that will be used to invoke it.
     */
    @Override
    public String getName() {
        return "countLiteralValues";
    }
    
    /**
     * Return the expected number of arguments for this functor or 0 if the number is flexible.
     */
    @Override
    public int getArgLength() {
        return 3;
    }

    /**
     * This method is invoked when the builtin is called in a rule body.
     * @param args the array of argument values for the builtin, this is an array 
     * of Nodes, some of which may be Node_RuleVariables.
     * @param length the length of the argument list, may be less than the length of the args array
     * for some rule engines
     * @param context an execution context giving access to other relevant data
     * @return return true if the buildin predicate is deemed to have succeeded in
     * the current environment
     */
    @Override
    public boolean bodyCall(Node[] args, int length, RuleContext context) {
        List<Node> values = new ArrayList<Node>();
        Node a0 = getArg(0, args, context);
        Node a1 = getArg(1, args, context);
        for (Iterator<Triple> ni = context.find(a0, a1, null); ni.hasNext(); ) {
            Node v = ni.next().getObject();
            if (v.isLiteral()) {
                // Can't just use contains because distinct objects may
                // be semantically equal
                boolean gotit = false;
                for (Iterator<Node> i = values.iterator(); i.hasNext(); ) {
                    if (v.sameValueAs(i.next())) {
                        gotit = true;
                        break;
                    }
                }
                if (!gotit) {
                    values.add(v);
                }
            }
        }
        return context.getEnv().bind(args[2], Util.makeIntNode(values.size()));
    }
    
}

==> jena-core-2.12.1-sources.jar: com/hp/hpl/jena/reasoner/rulesys/builtins/CountLiteralValues.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hp.hpl.jena.reasoner.rulesys.builtins;

import com.hp.hpl.jena.reasoner.rulesys.*;
import com.hp.hpl.jena.graph.*;
import java.util.*;

/**
 * CountLiteralValues(X, P, C) sets C to be the number of semantically
 * distinct values for P on resource X. This is expensive.
 */
public class CountLiteralValues extends BaseBuiltin {

    /**
     * Return a name for this builtin, normally this will be the name of the 
     * functor that will be used to invoke it.
     */
    @Override
    public String getName() {
        return "countLiteralValues";
    }
    
    /**
     * Return the expected number of arguments for this functor or 0 if the number is flexible.
     */
    @Override
    public int getArgLength() {
        return 3;
    }

    /**
     * This method is invoked when the builtin is called in a rule body.
     * @param args the array of argument values for the builtin, this is an array 
     * of Nodes, some of which may be Node_RuleVariables.
     * @param length the length of the argument list, may be less than the length of the args array
     * for some rule engines
     * @param context an execution context giving access to other relevant data
     * @return return true if the buildin predicate is deemed to have succeeded in
     * the current environment
     */
    @Override
    public boolean bodyCall(Node[] args, int length, RuleContext context) {
        List<Node> values = new ArrayList<>();
        Node a0 = getArg(0, args, context);
        Node a1 = getArg(1, args, context);
        for (Iterator<Triple> ni = context.find(a0, a1, null); ni.hasNext(); ) {
            Node v = ni.next().getObject();
            if (v.isLiteral()) {
                // Can't just use contains because distinct objects may
                // be semantically equal
                boolean gotit = false;
                for ( Node value : values )
                {
                    if ( v.sameValueAs( value ) )
                    {
                        gotit = true;
                        break;
                    }
                }
                if (!gotit) {
                    values.add(v);
                }
            }
        }
        return context.getEnv().bind(args[2], Util.makeIntNode(values.size()));
    }
    
}
